/**
 * MyDomparser2.java
 * Updated 11-11-2002
 */

package xml;

import java.io.*;
import futils.*;
import java.util.Vector;
import java.util.Enumeration;

// JAXP
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

// DOM
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Attr;
import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.DOMException;

// SAX Exceptions
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * JAXP DomParser program.
 * Demonstrates how to parse an xml file using the DOM api and JAXP.
 * This program uses JAXP 1.2. 
 * @author Thomas Rowland
 */
public class MyDomParser2 {

    private PrintStream out = null;
    private Document domDoc = null;
    private Element root = null;

    final int ELEMENT_NODE 	= 1;
    final int ATTR_NODE 	= 2;
    final int TEXT_NODE 	= 3;

    /** Constants used for JAXP 1.2 */
    static final String JAXP_SCHEMA_LANGUAGE =
    "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
    static final String W3C_XML_SCHEMA =
    "http://www.w3.org/2001/XMLSchema";    
	
    /**
     *	Parses an XML Document
     *	@param uri the uri of the xml source
     *	@param os the OutputStream to send the output to
     */
    public Document parse (String uri, OutputStream os) throws Exception {

        //-- set features -------------
        // (3) and (4)
        boolean dtdValidate = false;
        boolean xsdValidate = false;   

        try {
            out = new PrintStream (os);
            DocumentBuilderFactory dbf = 
                DocumentBuilderFactory.newInstance ();

            dbf.setIgnoringElementContentWhitespace (false);            

            // Set the validation mode to either: no validation, DTD
            // validation, or XSD validation
            dbf.setValidating(dtdValidate || xsdValidate);
            if (xsdValidate) {
                dbf.setNamespaceAware (true);
                dbf.setAttribute(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA);
            }
                        
            DocumentBuilder db = dbf.newDocumentBuilder ();
            db.setErrorHandler (new MyErrorHandler ());
            domDoc = db.parse (uri);
            root = domDoc.getDocumentElement ();
            traverse (root);
            return domDoc;
        }
        catch (FactoryConfigurationError e) {
                // Unable to create the factory
                throw new Exception (
                        "** FactoryConfigurationError\n" 
                        + e.getMessage ());
        }
        catch (ParserConfigurationException e) {
                // Factory unable to create the parser.
                throw new Exception (
                        "** ParserConfigurationException\n" 
                        + e.getMessage ());
        }
        catch (SAXException e) {
            // Exceptions thrown by the parser
            // Get the wrapped exception, if any
            Exception ex = e.getException ();
            String msg = null;
            if (ex != null) {
                ex.printStackTrace ();
                msg = ex.getMessage ();
            }
                throw new Exception (
                        "** SAXException:\n" + e.getMessage ());
        }
        catch (IOException e) {
                throw new Exception (
                        "IOException:\n" + e.getMessage ());
        }	
    }//parse
	

    /**
     *	Recursive method which traverses the DOM Document
     *	and outputs the content.
     *	@param elem the root element to begin recursion on
     */
    private void traverse (Node elem) {
        try {
            // handle the attributes
            if (elem.hasAttributes ()) {
                NamedNodeMap attrs = elem.getAttributes ();
                int alength = attrs.getLength ();

                // loop
                for (int i=0; i<alength; i++) {
                    Attr attr = (Attr) attrs.item (i);
                    out.println (attr.getName () + ":\t" + attr.getValue ());
                }
            }
            // handle the child nodes
            if (elem.hasChildNodes ()) {
                NodeList children = elem.getChildNodes ();
                int length = children.getLength ();

                // loop
                for (int i=0; i<length; i++) {
                    Node n = children.item (i);
                    String name = n.getNodeName ();

                    if (n.getNodeType () == ELEMENT_NODE) {
                        if (name.equals ("Item"))
                            out.println ("\n");
                        else
                            out.print (name + ":\t");
                        // recurse
                        traverse (n);
                    }
                    else
                    if (n.getNodeType () == TEXT_NODE) {
                        String txt = n.getNodeValue ().trim ();
                        if (! txt.equals ("")) {
                            out.println (txt);
                        }
                    }
                }
                return;
            }
        }
        catch (DOMException e) {
                System.out.println ("*** DOMException\n"
                    + e.getMessage ());
        }
    }//traverse
	

    /**
     *	Creates and returns a new Item object
     *	@return the new Item
     */
    public Item createItem () {
        return new Item ();
    }


    /**
     *	Adds a newly created Item object to the DOM Document
     *	@param item the Item object
     */
    public void addItem (Item item) throws Exception {
        Element itemElem = null;
        Element elem = null;
        Text txtNode = null;

        itemElem = domDoc.createElement ("Item");
        itemElem.setAttribute ("media", item.getMedia ());
        root.appendChild (itemElem);

        elem = domDoc.createElement ("Artist");
        txtNode = domDoc.createTextNode (item.getArtist ());
        elem.appendChild (txtNode);
        itemElem.appendChild (elem);

        elem = domDoc.createElement ("Title");
        txtNode = domDoc.createTextNode (item.getTitle ());
        elem.appendChild (txtNode);
        itemElem.appendChild (elem);

        elem = domDoc.createElement ("Year");
        txtNode = domDoc.createTextNode (item.getYear ());
        elem.appendChild (txtNode);
        itemElem.appendChild (elem);

        // Write code to add the members element


        traverse (root);
    }//addItem
	

    /**
     *	Writes a DOM Document to an OutputStream
     */
    public void toHtml (File xmlFile, File stylesheet) {
        try {
            String fn = xmlFile.getName ();
            fn = fn.substring (0, fn.lastIndexOf ("."));
            File htmlFile = new File (
                xmlFile.getParent () + "\\" + fn + ".html");
            System.out.println ("\nOutput to " + htmlFile);
            OutputStream out = new FileOutputStream (htmlFile);

            //Document document = parse (xmlFile, out);   			
            XmlTransformer.transform (domDoc, stylesheet, out);
            out.flush ();
            out.close ();
            System.out.println ("done.");

        } catch (IOException e) {
            e.printStackTrace();
        }
    }
	

    public void toXml (OutputStream _out) 
        throws Exception {
        XmlTransformer.writeXml (domDoc, _out);
    }

    /**
     *	Inner class
     * 	Error Handler to report validation errors and warnings.
     *	DOM uses these SAX error handlers.
     */
    private class MyErrorHandler implements ErrorHandler {

        //-- The following methods are standard SAX ErrorHandler methods.
        public void warning (SAXParseException spe) throws SAXException {
            System.out.println ("Warning: " + getParseExceptionInfo(spe));
        }
        
        public void error (SAXParseException spe) throws SAXException {
            String message = "Error: " + getParseExceptionInfo(spe);
            throw new SAXException (message);
        }

        public void fatalError (SAXParseException spe) throws SAXException {
            String message = "Fatal Error: " + getParseExceptionInfo(spe);
            throw new SAXException (message);
        }
        
        /**
         * Returns a string describing parse exception details
         */
        private String getParseExceptionInfo (SAXParseException spe) {
            String systemId = spe.getSystemId ();
            if (systemId == null) {
                systemId = "null";
            }
            String info = "URI=" + systemId +
                "\nLine=" + spe.getLineNumber () +
                "\n" + spe.getMessage();
            return info;
        }
    }

}//class

