/**
 * XmlTransformer.java
 * Updated 11-11-2002
 */

package xml;

import java.io.File;
import java.io.OutputStream;
import java.io.IOException;

// DOM
import org.w3c.dom.Document;

// for XSLT transformation
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.stream.StreamResult;


/**
 * This object uses XSLT to write out a DOM document to an OutputStream.
 * This program uses the DOM Level 2 api and JAXP 1.2. 
 */
public class XmlTransformer {

	/**
	 * Private constructor - no instances of this class should be created.
	 * All public methods are decared as static.
	 */
	private XmlTransformer () {	}
	
	
	/**
	 * Writes a DOM Document to an OutputStream
	 * @param node the top level node of the DOM tree
	 * @param out OutputStream to write the output to
	 */
	public static void writeXml (Document document, OutputStream out) 
		throws Exception {
		try {
                    // Obtain a Transformer
                    TransformerFactory tf = TransformerFactory.newInstance ();
                    Transformer t = tf.newTransformer ();
                    
                    // Create a source object
                    DOMSource source = 
                        new DOMSource (document.getDocumentElement ());

                    // Create a result object and perform the transform
                    StreamResult result = new StreamResult (out);
                    t.transform (source, result);
		} 
		catch (TransformerConfigurationException e) {
                    // Exception generated by the TransformerFactory
		    Throwable ex = e.getException ();
		    String msg = null;
		    if (ex != null) {
		    	ex.printStackTrace ();
		    	msg = ex.getMessage ();
		    }
			throw new Exception (
				"** TransformerConfigurationException:\n" 
				+ e.getMessage ());
		} 
		catch (TransformerException e) {
                    // Exception generated by the Transformer
		    Throwable ex = e.getException ();
		    String msg = null;
		    if (ex != null) {
		    	ex.printStackTrace ();
		    	msg = ex.getMessage ();
		    }
			throw new Exception (
				"** TransformerException:\n" 
				+ e.getMessage ());
		}
	}//writeXml	
	

	/**
	 * Transforms a DOM Document into another form, s
         * pecified by its stylesheet.
	 * @param node the top level node of the DOM tree
	 * @param stylesheet the stylesheet containing the transform defs
	 * @param out OutputStream to write the output to
	 */
	public static void transform (Document document, 
                                      File stylesheet, 
                                      OutputStream out) {
            try {
                // Instantiate the TransformerFactory
                TransformerFactory tf = TransformerFactory.newInstance ();

                // Create a source object from the stylesheet
                StreamSource ss = new StreamSource (stylesheet);

                // Obtain a Transformer
                Transformer t = tf.newTransformer (ss);

                // Create a source object from the DOM document
                DOMSource ds = new DOMSource (document.getDocumentElement ());

                // Create a result object from the BufferedWriter
                StreamResult result = new StreamResult (out);

                // Perform the transform
                t.transform (ds, result);
            } 
            catch (TransformerConfigurationException tce) {
                // Exception generated by the TransformerFactory
                System.out.println ("\n** Transformer Factory error");
                System.out.println (" " + tce.getMessage() );
                Throwable x = tce;
                if (tce.getException () != null)
                    x = tce.getException ();
                x.printStackTrace ();
            } 
            catch (TransformerException te) {
                // Exception generated by the transformer
                System.out.println ("\n** Transformation error");
                System.out.println(" " + te.getMessage() );
                Throwable x = te;
                if (te.getException() != null)
                    x = te.getException();
                x.printStackTrace();
            }

	}//transform

}//class